package Hooke;

/**
 *
 * @author Danilo
 */
public class Vector2D {
    private double compX;
    private double compY;

/**
 * Constructor de la clase, para iniciar un vector con su <b>radio</b> y <b>angulo</b>
 * @param radio radio del nuevo vector
 * @param theta angulo del nuevo vector
 */
    public Vector2D(double radio, double theta){
        this.compX= radio * Math.cos(theta);
        this.compY= radio * Math.sin(theta);
    }
/**
 * Constructor de la clase, con cordenadas <b>X</b> e <b>Y</b>
 * @param x
 * @param y
 */
     public Vector2D(float x, float y){
        this.compX= x;
        this.compY= y;
    }
/**
 * Calcula la distancia entre este vector (o la posicion) y otro
 * @param vector El otro vector
 * @return La distancia entre los dos vectores
 */
    public double distanciaVector(Vector2D vector){
        return Math.sqrt(Math.pow(vector.getComponentX() - this.compX,2)+Math.pow(vector.getComponentY() - this.compY,2));
    }
/**
 * Calcula la distancia entre los dos vectores, o las dos puntas de los vectores
 * @param vector1 Primer Vector
 * @param vector2 Segundo Vector
 * @return Distancia entre vectores
 */
    public static double distanciaVectores(Vector2D vector1, Vector2D vector2){
        if(vector1 == null) return vector2.getRadio();
        else if(vector2 == null) return vector1.getRadio();
        else return Math.sqrt(Math.pow(vector1.getComponentX() - vector2.getComponentX(),2)+Math.pow(vector1.getComponentY() - vector2.getComponentY(),2));
    }
/**
 * Devuelve la componente X de este vector
 * @return la Componente X de este Vector
 */
    public double getComponentX(){
        return this.compX;
    }
/**
 * Devuelve la componente Y de este vector
 * @return la Componente Y de este Vector
 */
    public double getComponentY(){
        return this.compY;
    }
 /**
 * Devuelve el angulo de este vector
 * @return el angulo de este Vector
 */
    public double getAngulo(){
        return Math.atan2(compY, compX);
    }
/**
 * Setea el angulo de este Vector
 * @param angulo Nuevo angulo
 */
    public void setAngulo(double angulo){
        double radio=Math.sqrt(Math.pow(this.compX,2)+Math.pow(this.compY,2));
        this.compX = radio*Math.cos(angulo);
        this.compY = radio*Math.sin(angulo);
    }
/**
 * Devuelve el radio de este vector
 * @return El radio de este vector
 */
    public double getRadio(){
        return Math.sqrt(Math.pow(this.compX,2)+Math.pow(this.compY,2));
    }
/**
 * Devuelve el angulo entre dos vectores
 * @param pos1 primer vector
 * @param pos2 segundo vector
 * @return angulo entre vectores
 */
    public static double getAngulo(Vector2D pos1,Vector2D pos2){
        if(pos1 == null || pos2 == null) return 0;
        return Math.atan2(pos1.getComponentY()-pos2.getComponentY(), pos1.getComponentX()-pos2.getComponentX());
    }
/**
 * Multiplica este vector por un escalar
 * @param escalar numero escalar por el cual sera multiplicado el vector
 */
    public void multiplicaEscalar(double escalar){
        double radio = Math.sqrt(Math.pow(this.compX,2)+Math.pow(this.compY,2));
        double angulo = Math.atan2(compY, compX);
        radio*=escalar;
        this.compX = radio*Math.cos(angulo);
        this.compY = radio*Math.sin(angulo);
    }
/**
 * Setea las cordenadas X e Y de este vector
 * @param x nueva cordenada X
 * @param y nueva cordenada Y
 */
    public void setXY(double x, double y){
        this.compX = x;
        this.compY = y;
    }
/**
 * Suma a este vector otro
 * @param vector vector a sumar
 */
    public void sumarVector(Vector2D vector){
        this.compX+=vector.getComponentX();
        this.compY+=vector.getComponentY();
    }
/**
 * Resta a este vector otro
 * @param vector vector a restar
 */
    public void restarVector(Vector2D vector){
        Vector2D vectorNegativo = new Vector2D((float)-vector.getComponentX(),(float)-vector.getComponentY());
        this.sumarVector(vectorNegativo);
    }
/**
 * Resta dos vectores
 * @param vector1 Primer Vector
 * @param vector2 Segundo Vector
 * @return Vector resultante de la resta
 */
    public static Vector2D restarVectores(Vector2D vector1, Vector2D vector2){
        Vector2D vectorNegativo = new Vector2D((float)-vector2.getComponentX(),(float)-vector2.getComponentY());
        Vector2D vector = new Vector2D((float)vector1.getComponentX(),(float)vector1.getComponentY());
        vector.sumarVector(vectorNegativo);
        return vector;
    }
/**
 * Incrementa un vector en "sumando" unidades
 * @param vector Vector a incrementar
 * @param sumando Lo que se le sumara al vector
 * @return
 */
    public static Vector2D sumarRadio(Vector2D vector, double sumando){
        Vector2D vector1 = new Vector2D(vector.getRadio()+sumando,vector.getAngulo());
        return vector1;
    }
/**
 * Devuelve un string con las cordenadas X e Y
 * @return String con cordenadas X e Y
 */
    @Override
    public String toString(){
        return this.compX+","+this.compY;
    }
}

